@@ -44,12 +44,11 @@
 #include "opkg_verify.h"
 #include "xsystem.h"
 #include "xfuncs.h"
-
-#ifdef HAVE_SOLVER
 #include "opkg_solver.h"
-#else
-#include "opkg_action.h"
-#include "opkg_upgrade.h"
+
+/* Needed by opkg_list_upgradable */
+#ifdef HAVE_SOLVER_INTERNAL
+#include "solvers/internal/opkg_upgrade_internal.h"
 #endif
 
 static void print_pkg(pkg_t * pkg)
@@ -180,7 +179,7 @@ static opkg_intercept_t opkg_prep_intercepts(void)
 
     ctx = xcalloc(1, sizeof(*ctx));
     ctx->oldpath = xstrdup(getenv("PATH"));
-    sprintf_alloc(&newpath, "%s/opkg/intercept:%s", DATADIR, ctx->oldpath);
+    sprintf_alloc(&newpath, "%s:%s", opkg_config->intercepts_dir, ctx->oldpath);
     sprintf_alloc(&ctx->statedir, "%s/opkg-intercept-XXXXXX",
                   opkg_config->tmp_dir);
 
@@ -196,6 +195,7 @@ static opkg_intercept_t opkg_prep_intercepts(void)
 
     setenv("OPKG_INTERCEPT_DIR", ctx->statedir, 1);
     setenv("PATH", newpath, 1);
+    opkg_msg(DEBUG, "Added intercepts dir to PATH; new PATH=%s\n", newpath);
     free(newpath);
 
     return ctx;
@@ -207,6 +207,7 @@ static int opkg_finalize_intercepts(opkg_intercept_t ctx)
     int err = 0;
 
     setenv("PATH", ctx->oldpath, 1);
+    opkg_msg(DEBUG, "Removed intercepts dir from PATH; old PATH=%s\n", ctx->oldpath);
     free(ctx->oldpath);
 
     dir = opendir(ctx->statedir);
@@ -221,6 +222,7 @@ static int opkg_finalize_intercepts(opkg_intercept_t ctx)
             sprintf_alloc(&path, "%s/%s", ctx->statedir, de->d_name);
             if (access(path, X_OK) == 0) {
                 const char *argv[] = { "sh", "-c", path, NULL };
+                opkg_msg(DEBUG, "Run intercepted script %s\n", path);
                 xsystem(argv);
             }
             free(path);
@@ -416,11 +418,7 @@ static int opkg_install_cmd(int argc, char **argv)
     }
     pkg_info_preinstall_check();
 
-#ifdef HAVE_SOLVER
     err = opkg_solver_install(argc, argv);
-#else
-    err = opkg_install(argc, argv);
-#endif
 
     r = opkg_configure_packages(NULL);
     if (r != 0)
@@ -440,11 +438,7 @@ static int opkg_upgrade_cmd(int argc, char **argv)
 
     pkg_info_preinstall_check();
 
-#ifdef HAVE_SOLVER
-    opkg_solver_upgrade(argc, argv);
-#else
-    err = opkg_upgrade(argc, argv);
-#endif
+    err = opkg_solver_upgrade(argc, argv);
 
     r = opkg_configure_packages(NULL);
     if (r != 0)
@@ -571,7 +565,7 @@ static int opkg_list_changed_conffiles_cmd(int argc, char **argv)
 
 static int opkg_list_upgradable_cmd(int argc, char **argv)
 {
-#ifdef HAVE_SOLVER
+#ifndef HAVE_SOLVER_INTERNAL
     // TODO: this requires prepare_upgrade_list in opkg_upgrade.c but opkg_upgrade
     // is not built when an external solver is enabled. Either let the solver
     // handle this, include opkg_upgrade, or put prepare_upgrade_list somewhere else.
@@ -692,11 +686,7 @@ static int opkg_remove_cmd(int argc, char **argv)
 
     pkg_info_preinstall_check();
 
-#ifdef HAVE_SOLVER
     err = opkg_solver_remove(argc, argv);
-#else
-    err = opkg_remove(argc, argv);
-#endif
 
     write_status_files_if_changed();
     return err;
@@ -839,17 +829,8 @@ static int pkg_mark_provides(pkg_t * pkg)
     return 0;
 }
 
-enum what_field_type {
-    WHATDEPENDS,
-    WHATCONFLICTS,
-    WHATPROVIDES,
-    WHATREPLACES,
-    WHATRECOMMENDS,
-    WHATSUGGESTS
-};
-
-static int opkg_what_depends_conflicts_cmd(enum depend_type what_field_type,
-                                           int recursive, int argc, char **argv)
+static int opkg_what_depends_conflicts_replaces_cmd(enum depend_type what_field_type,
+                                                    int recursive, int argc, char **argv)
 {
     depend_t *possibility;
     compound_depend_t *cdep;
@@ -874,6 +855,9 @@ static int opkg_what_depends_conflicts_cmd(enum depend_type what_field_type,
     case RECOMMEND:
         rel_str = "recommends";
         break;
+    case REPLACES:
+        rel_str = "replaces";
+        break;
     default:
         return -1;
     }
@@ -907,14 +891,16 @@ static int opkg_what_depends_conflicts_cmd(enum depend_type what_field_type,
         for (j = 0; j < available_pkgs->len; j++) {
             pkg = available_pkgs->pkgs[j];
             count = (what_field_type == CONFLICTS) ? pkg->conflicts_count :
-                    (pkg->pre_depends_count + pkg->depends_count + pkg->recommends_count + pkg->suggests_count);
+                    ((what_field_type == REPLACES) ? pkg->replaces_count :
+                    (pkg->pre_depends_count + pkg->depends_count + pkg->recommends_count + pkg->suggests_count));
 
             /* skip this package if it is already marked */
             if (pkg->parent->state_flag & SF_MARKED)
                 continue;
 
             for (k = 0; k < count; k++) {
-                cdep = (what_field_type == CONFLICTS) ? &pkg->conflicts[k] : &pkg->depends[k];
+                cdep = (what_field_type == CONFLICTS) ? &pkg->conflicts[k] :
+                       ((what_field_type == REPLACES) ? &pkg->replaces[k] : &pkg->depends[k]);
 
                 if (what_field_type != cdep->type)
                     continue;
@@ -958,35 +944,33 @@ static int opkg_what_depends_conflicts_cmd(enum depend_type what_field_type,
 
 static int opkg_whatdepends_recursively_cmd(int argc, char **argv)
 {
-    return opkg_what_depends_conflicts_cmd(DEPEND, 1, argc, argv);
+    return opkg_what_depends_conflicts_replaces_cmd(DEPEND, 1, argc, argv);
 }
 
 static int opkg_whatdepends_cmd(int argc, char **argv)
 {
-    return opkg_what_depends_conflicts_cmd(DEPEND, 0, argc, argv);
+    return opkg_what_depends_conflicts_replaces_cmd(DEPEND, 0, argc, argv);
 }
 
 static int opkg_whatsuggests_cmd(int argc, char **argv)
 {
-    return opkg_what_depends_conflicts_cmd(SUGGEST, 0, argc, argv);
+    return opkg_what_depends_conflicts_replaces_cmd(SUGGEST, 0, argc, argv);
 }
 
 static int opkg_whatrecommends_cmd(int argc, char **argv)
 {
-    return opkg_what_depends_conflicts_cmd(RECOMMEND, 0, argc, argv);
+    return opkg_what_depends_conflicts_replaces_cmd(RECOMMEND, 0, argc, argv);
 }
 
 static int opkg_whatconflicts_cmd(int argc, char **argv)
 {
-    return opkg_what_depends_conflicts_cmd(CONFLICTS, 0, argc, argv);
+    return opkg_what_depends_conflicts_replaces_cmd(CONFLICTS, 0, argc, argv);
 }
 
-static int opkg_what_provides_replaces_cmd(enum what_field_type what_field_type,
-                                           int argc, char **argv)
+static int opkg_whatprovides_cmd(int argc, char **argv)
 {
     if (argc > 0) {
         pkg_vec_t *available_pkgs = pkg_vec_alloc();
-        const char *rel_str = (what_field_type == WHATPROVIDES ? "provides" : "replaces");
         int i;
 
         pkg_info_preinstall_check();
@@ -999,17 +983,17 @@ static int opkg_what_provides_replaces_cmd(enum what_field_type what_field_type,
             const char *target = argv[i];
             unsigned int j;
 
-            opkg_msg(NOTICE, "What %s %s\n", rel_str, target);
+            opkg_msg(NOTICE, "What provides %s\n", target);
             for (j = 0; j < available_pkgs->len; j++) {
                 pkg_t *pkg = available_pkgs->pkgs[j];
                 int k;
-                int count = (what_field_type == WHATPROVIDES) ? pkg->provides_count : pkg->replaces_count;
+                int count = pkg->provides_count;
                 for (k = 0; k < count; k++) {
-                    abstract_pkg_t *apkg = ((what_field_type == WHATPROVIDES) ? pkg->provides[k] : pkg->replaces[k]);
+                    abstract_pkg_t *apkg = pkg->provides[k];
                     if (fnmatch(target, apkg->name, 0) == 0) {
                         opkg_msg(NOTICE, "    %s", pkg->name);
                         if (strcmp(target, apkg->name) != 0)
-                            opkg_msg(NOTICE, "\t%s %s\n", rel_str, apkg->name);
+                            opkg_msg(NOTICE, "\tprovides %s\n", apkg->name);
                         opkg_message(NOTICE, "\n");
                     }
                 }
@@ -1020,14 +1004,9 @@ static int opkg_what_provides_replaces_cmd(enum what_field_type what_field_type,
     return 0;
 }
 
-static int opkg_whatprovides_cmd(int argc, char **argv)
-{
-    return opkg_what_provides_replaces_cmd(WHATPROVIDES, argc, argv);
-}
-
 static int opkg_whatreplaces_cmd(int argc, char **argv)
 {
-    return opkg_what_provides_replaces_cmd(WHATREPLACES, argc, argv);
+    return opkg_what_depends_conflicts_replaces_cmd(REPLACES, 0, argc, argv);
 }
 
 static int opkg_search_cmd(int argc, char **argv)
@@ -1075,7 +1054,8 @@ static int opkg_compare_versions_cmd(int argc, char **argv)
         struct pkg p1, p2;
         parse_version(&p1, argv[0]);
         parse_version(&p2, argv[2]);
-        return pkg_version_satisfied(&p1, &p2, argv[1]);
+        p1.force_reinstall = p2.force_reinstall = 0;
+        return pkg_version_satisfied(&p1, &p2, argv[1]) ? 0 : 1;
     } else {
         opkg_msg(ERROR,
                  "opkg compare_versions <v1> <op> <v2>\n"
@@ -1129,10 +1109,8 @@ static opkg_cmd_t cmds[] = {
         PFM_DESCRIPTION | PFM_SOURCE},
     {"download", 1, (opkg_cmd_fun_t) opkg_download_cmd,
         PFM_DESCRIPTION | PFM_SOURCE},
-    {"compare_versions", 1, (opkg_cmd_fun_t) opkg_compare_versions_cmd,
-        PFM_DESCRIPTION | PFM_SOURCE},
-    {"compare-versions", 1, (opkg_cmd_fun_t) opkg_compare_versions_cmd,
-        PFM_DESCRIPTION | PFM_SOURCE},
+    {"compare_versions", 1, (opkg_cmd_fun_t) opkg_compare_versions_cmd, 0},
+    {"compare-versions", 1, (opkg_cmd_fun_t) opkg_compare_versions_cmd, 0},
     {"print-architecture", 0, (opkg_cmd_fun_t) opkg_print_architecture_cmd,
         PFM_DESCRIPTION | PFM_SOURCE},
     {"print_architecture", 0, (opkg_cmd_fun_t) opkg_print_architecture_cmd,
