@@ -34,12 +34,6 @@
 #include "file_util.h"
 #include "xfuncs.h"
 
-void pkg_hash_init(void)
-{
-    hash_table_init("pkg-hash", &opkg_config->pkg_hash,
-                    OPKG_CONF_DEFAULT_HASH_LEN);
-}
-
 static void free_pkgs(const char *key, void *entry, void *data)
 {
     unsigned int i;
@@ -66,40 +60,7 @@ static void free_pkgs(const char *key, void *entry, void *data)
     free(ab_pkg);
 }
 
-void pkg_hash_deinit(void)
-{
-    hash_table_foreach(&opkg_config->pkg_hash, free_pkgs, NULL);
-    hash_table_deinit(&opkg_config->pkg_hash);
-}
-
-int dist_hash_add_from_file(pkg_src_t * dist)
-{
-    nv_pair_list_elt_t *l;
-    char *list_file, *subname;
-    int r;
-
-    list_for_each_entry(l, &opkg_config->arch_list.head, node) {
-        nv_pair_t *nv = (nv_pair_t *) l->data;
-        sprintf_alloc(&subname, "%s-%s", dist->name, nv->name);
-        sprintf_alloc(&list_file, "%s/%s", opkg_config->lists_dir, subname);
-
-        if (file_exists(list_file)) {
-            r = pkg_hash_add_from_file(list_file, dist, NULL, 0);
-            if (r != 0) {
-                free(list_file);
-                return -1;
-            }
-            pkg_src_list_append(&opkg_config->pkg_src_list, subname,
-                                dist->value, "__dummy__", 0);
-        }
-
-        free(list_file);
-    }
-
-    return 0;
-}
-
-int pkg_hash_add_from_file(const char *file_name, pkg_src_t * src,
+static int pkg_hash_add_from_file(const char *file_name, pkg_src_t * src,
                            pkg_dest_t * dest, int is_status_file)
 {
     pkg_t *pkg;
@@ -114,6 +75,10 @@ int pkg_hash_add_from_file(const char *file_name, pkg_src_t * src,
         return -1;
     }
 
+    /* Remove UTF-8 BOM if present */
+    if (!(getc(fp) == 0xEF && getc(fp) == 0xBB && getc(fp) == 0xBF))
+        rewind(fp);
+
     buf = xmalloc(len);
 
     do {
@@ -169,6 +134,45 @@ int pkg_hash_add_from_file(const char *file_name, pkg_src_t * src,
     return ret;
 }
 
+static int dist_hash_add_from_file(pkg_src_t * dist)
+{
+    nv_pair_list_elt_t *l;
+    char *list_file, *subname;
+    int r;
+
+    list_for_each_entry(l, &opkg_config->arch_list.head, node) {
+        nv_pair_t *nv = (nv_pair_t *) l->data;
+        sprintf_alloc(&subname, "%s-%s", dist->name, nv->name);
+        sprintf_alloc(&list_file, "%s/%s", opkg_config->lists_dir, subname);
+
+        if (file_exists(list_file)) {
+            r = pkg_hash_add_from_file(list_file, dist, NULL, 0);
+            if (r != 0) {
+                free(list_file);
+                return -1;
+            }
+            pkg_src_list_append(&opkg_config->pkg_src_list, subname,
+                                dist->value, "__dummy__", 0);
+        }
+
+        free(list_file);
+    }
+
+    return 0;
+}
+
+void pkg_hash_init(void)
+{
+    hash_table_init("pkg-hash", &opkg_config->pkg_hash,
+                    OPKG_CONF_DEFAULT_HASH_LEN);
+}
+
+void pkg_hash_deinit(void)
+{
+    hash_table_foreach(&opkg_config->pkg_hash, free_pkgs, NULL);
+    hash_table_deinit(&opkg_config->pkg_hash);
+}
+
 /*
  * Load in feed files from the cached "src" and/or "src/gz" locations.
  */
@@ -339,7 +343,7 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(abstract_pkg_t * apkg,
             if (abstract_pkg_vec_contains(providers, replacement_apkg))
                 continue;
             else
-                provider_apkg = replacement_apkg;
+                abstract_pkg_vec_insert(providers, replacement_apkg);
         }
 
         vec = provider_apkg->pkgs;
