@@ -24,4 +24,4 @@ if not opkgcl.is_installed("x"):
 
 opkgcl.install("y")
 if not opkgcl.is_installed("y"):
-	opk.xfail("Package 'y' not installed despite lack of conflicts.")
+	opk.fail("Package 'y' not installed despite lack of conflicts.")
